from ..enums import MeshLevel
from ..models import Indicator
from ..payloads import Territory
from .commons import (
    add_optional_filters,
    calculate_aggregate_values,
    get_last_year,
    get_place_names_join,
    get_where_territory,
)
from .utils import get_breakdown_dimension, run_custom_query


def get_proportions_chart(indicator, territory, filters):
    where_territory = get_where_territory(territory)
    last_year = get_last_year(indicator, territory.mesh)
    breakdown_dimension = get_breakdown_dimension(indicator)

    query = f"""
    SELECT {calculate_aggregate_values(indicator, with_alternative=False)}, "{breakdown_dimension.db_name}" as dimension
    FROM "{indicator.db_table_prefix}_{territory.mesh}" as indic
    WHERE {where_territory} AND annee = {last_year}
    {add_optional_filters(indicator, filters)}
    GROUP BY "{breakdown_dimension.db_name}"
"""

    data = run_custom_query(query)
    filters_color = {f.db_name: f.color for f in breakdown_dimension.filters.all()}
    data_dict = {
        d["dimension"]: {
            "label": d["dimension"],
            "data": [d["valeur"]],
            "color": filters_color.get(d["dimension"]),
        }
        for d in data
    }
    breakdown_filters = filters[breakdown_dimension.db_name]
    sorted_data = (
        [data_dict[filter] for filter in breakdown_filters if filter in data_dict]
        if filters
        else list(data_dict.values())
    )
    return sorted_data


def get_values_for_submesh_territories(
    indicator: Indicator, submesh: MeshLevel, territory: Territory, filters
):
    submesh = submesh.lower()
    territory_mesh = "DEPCOM" if territory.mesh == "com" else territory.mesh.upper()
    mapped_submesh = "DEPCOM" if submesh == "com" else submesh.upper()

    geo_ids = "', '".join([id.strip() for id in territory.id.split(",")])
    query = f"""
    SELECT {calculate_aggregate_values(indicator)}, code_{submesh} as geocode, arbo.lieu as geoname
    FROM "{indicator.db_table_prefix}_{submesh}" indic
    {get_place_names_join(True, mapped_submesh, submesh)}
    WHERE code_{submesh} IN (
        SELECT DISTINCT("{mapped_submesh}")
        FROM arborescence_geo arbo
        WHERE arbo."{territory_mesh}" IN ('{geo_ids}')
    )
    {add_optional_filters(indicator, filters)}
    AND annee = (
        SELECT MAX(annee)
        FROM "{indicator.db_table_prefix}_{submesh}"
    )
    GROUP BY geocode, geoname
    """
    return run_custom_query(query)



from django.contrib import admin
from django.contrib.auth.models import Group

from territories_dashboard_lib.website_lib.forms import (
    GlossaryItemAdminForm,
    LandingPageAdminForm,
    MainConfAdminForm,
    StaticPageAdminForm,
)
from territories_dashboard_lib.website_lib.models import (
    GlossaryItem,
    LandingPage,
    MainConf,
    NoticeBanner,
    StaticPage,
)


@admin.register(MainConf)
class MainConfAdmin(admin.ModelAdmin):
    form = MainConfAdminForm


@admin.register(GlossaryItem)
class GlossaryItemAdmin(admin.ModelAdmin):
    form = GlossaryItemAdminForm


@admin.register(LandingPage)
class LandingPageAdminForm(admin.ModelAdmin):
    form = LandingPageAdminForm


@admin.register(StaticPage)
class StaticPageAdminForm(admin.ModelAdmin):
    form = StaticPageAdminForm


@admin.register(NoticeBanner)
class NoticeBannerAdmin(admin.ModelAdmin):
    list_display = ("title", "created_at")


admin.site.unregister(Group)



from collections import defaultdict

from .commons import add_optional_filters, calculate_aggregate_values
from .utils import run_custom_query


def get_comparison_values_and_buckets(
    indicator, submesh, territories, cmp_territories, filters
):
    territories_ids = [t["code"] for t in territories + cmp_territories]
    territories_sql = f"('{"', '".join(territories_ids)}')"
    table_name = f"{indicator.db_table_prefix}_{submesh}"
    filters = add_optional_filters(indicator, filters)
    value = calculate_aggregate_values(indicator, with_alternative=False)
    number_of_buckets = min(10, len(territories) + len(cmp_territories))
    sub_query = f"""
    SELECT
        {value},
        code_{submesh} as geo_code
    FROM {table_name} indic
    WHERE code_{submesh} IN {territories_sql}
    AND annee = (
        SELECT MAX(annee)
        FROM {table_name}
    )
    AND valeur IS NOT NULL
    {filters}
    GROUP BY geo_code
    """
    query = f"""
    WITH aggregated_data AS ({sub_query}),
    range_bounds AS (
        SELECT
            MIN(valeur) - 1e-10 AS min_value,
            MAX(valeur) + 1e-10 AS max_value
        FROM aggregated_data
    )
    SELECT
        aggregated_data.valeur,
        aggregated_data.geo_code,
        WIDTH_BUCKET(aggregated_data.valeur, range_bounds.min_value, range_bounds.max_value, {number_of_buckets}) AS bucket
    FROM aggregated_data
    CROSS JOIN
        range_bounds
    """
    results = run_custom_query(query)
    raw_values = [r["valeur"] for r in results]
    min_value = min(raw_values)
    max_value = max(raw_values)
    bucket_width = (max_value - min_value) / number_of_buckets
    buckets = []
    bucket_min = min_value
    for i in range(1, number_of_buckets + 1):
        bucket_max = (
            max_value if i == number_of_buckets else min_value + i * bucket_width
        )
        buckets.append([bucket_min, bucket_max])
        bucket_min = bucket_max
    values = defaultdict(list)
    cmp_values = defaultdict(list)
    territories_ids_set = {t["code"] for t in territories}
    cmp_territories_ids_set = {t["code"] for t in cmp_territories}
    territories_dict = {t["code"]: t["name"] for t in territories + cmp_territories}
    for r in results:
        geo_code = r["geo_code"]
        if geo_code in territories_ids_set:
            values[r["bucket"]].append(territories_dict[geo_code])
        if geo_code in cmp_territories_ids_set:
            cmp_values[r["bucket"]].append(territories_dict[geo_code])
    return values, cmp_values, buckets



from typing import List, Optional

from pydantic import BaseModel, field_validator

from territories_dashboard_lib.commons.types import TerritoryCode
from territories_dashboard_lib.indicators_lib.enums import (
    GeoLevel,
    MeshLevel,
)


class GeoFeaturesParams(BaseModel):
    mesh: MeshLevel
    geo_level: GeoLevel
    main_territories: List[TerritoryCode]
    last: Optional[int] = None
    limit: Optional[int] = 1000
    feature: int

    @field_validator("main_territories", mode="before")
    def split_main_territories(cls, v):
        if isinstance(v, str):
            return v.split(",")
        return v


class MainTerritoryParams(BaseModel):
    geo_level: GeoLevel
    geo_id: List[TerritoryCode]

    @field_validator("geo_id", mode="before")
    def split_main_territories(cls, v):
        if isinstance(v, str):
            return v.split(",")
        return v


class TerritoriesParams(BaseModel):
    mesh: MeshLevel
    territories: List[TerritoryCode]

    @field_validator("territories", mode="before")
    def split_main_territories(cls, v):
        if isinstance(v, str):
            return v.split(",")
        return v


class TerritoryFeatureParams(BaseModel):
    mesh: MeshLevel
    geo_level: GeoLevel | None = None
    main_territories: List[TerritoryCode] | None = None
    codes: List[TerritoryCode] | None = None

    @field_validator("main_territories", "codes", mode="before")
    def split_main_territories(cls, v):
        if isinstance(v, str):
            return v.split(",")
        return v


class SearchTerritoriesParams(BaseModel):
    mesh: MeshLevel
    search: str = ""
    offset: int = 0



# Generated by Django 5.1.6 on 2025-06-23 12:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('website_lib', '0002_mainconf_contact_email_mainconf_newsletter_link_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mainconf',
            name='footer_navigation',
            field=models.TextField(default='', help_text='Navigation du footer, mettre les liens au format markdown, un par ligne. <br/><br/>\n    Si le lien est interne, commencer et terminer le lien par un slash comme "/accueil/"<br/><br/>\n    Par exemple : <br/><br/>\n    [Plan du site](/plan-site/)<br/>\n    [Accessibilité](/accessibilite/)<br/>\n    [Mentions légales](/mentions-legales/) <br/>\n    ', verbose_name='Navigation du footer'),
        ),
        migrations.AlterField(
            model_name='mainconf',
            name='header_navigation',
            field=models.TextField(default='', help_text='Navigation du header, mettre les liens au format markdown avec un slash à la fin du lien, mettre une ligne vide entre chaque rubrique. <br/><br/>\n        \n        Une rubrique peut contenir un seul lien, ou bien plusiers, dans ce cas la rubrique sera sous forme de dropdown et il faut mettre un titre en premier. <br/><br/>Par exemple : <br/>\n        <br/><br/>\n        [Accueil](/accueil/)\n        <br/><br/>\n        [Indicateurs territoriaux](/indicateurs/)\n        <br/><br/>\n        À propos\n        [Présentation](/presentation/)\n        [Journal des versions](/journal/)\n        ', verbose_name='Navigation du header'),
        ),
    ]



from ..enums import (
    FRANCE_DB_VALUES,
    MeshLevel,
)
from ..models import AggregationFunctions, Indicator
from .utils import get_breakdown_dimension, run_custom_query


def get_last_year(indicator, mesh):
    query = f""" SELECT MAX(annee) as last_year FROM "{indicator.db_table_prefix}_{mesh}" """
    return run_custom_query(query)[0]["last_year"]


def generate_aggregate_query(indicator, territory, submesh, filters, slicer):
    query = f"""
    WITH annee_max AS
    (SELECT MAX(annee) FROM "{indicator.db_table_prefix}_{submesh}"),
    aggregat AS
    (
        SELECT
        {calculate_aggregate_values(indicator)}, {slicer}
        FROM
        {get_table_data_for_geography(indicator, territory, submesh)}
        {add_optional_filters(indicator, filters)}
        AND annee = (SELECT * FROM annee_max)
        GROUP BY (indic.{slicer})
    )
    """
    return query


def generate_aggregate_query_for_location(indicator, territory, submesh, filters):
    return generate_aggregate_query(
        indicator, territory, submesh, filters, f"code_{submesh}"
    )


def add_optional_filters(indicator: Indicator, filters):
    condition = ""
    all_dimensions = [dimension.db_name for dimension in indicator.dimensions.all()]
    for dimension in all_dimensions:
        if filters and filters.get(dimension):
            filters_str = ", ".join(
                [f"'{value.replace("'", "''")}'" for value in filters.get(dimension)]
            )
            condition += f' AND indic."{dimension}" in ({filters_str}) '
    return condition


def get_place_names_join(include_place_name, mapped_submesh, submesh, flows=False):
    join_clause = ""
    if include_place_name:
        if flows:
            join_clause = f"""
            JOIN
                (SELECT distinct("NOM_{mapped_submesh}") as territory_1, "{mapped_submesh}" as territory_1_id, "{mapped_submesh}" from arborescence_geo) arbo1
                on arbo1."{mapped_submesh}" = indic.code_{submesh.lower()}_1
            JOIN
                (SELECT distinct("NOM_{mapped_submesh}") as territory_2, "{mapped_submesh}" as territory_2_id, "{mapped_submesh}" from arborescence_geo) arbo2
                on arbo2."{mapped_submesh}" = indic.code_{submesh.lower()}_2
            """
        else:
            join_clause = f"""
                JOIN
                (SELECT distinct("NOM_{mapped_submesh}") as lieu, "{mapped_submesh}" as territoryid, "{mapped_submesh}" from arborescence_geo) arbo
                on arbo."{mapped_submesh}" = indic.code_{submesh.lower()}
            """
    return join_clause


def get_table_data_for_geography(
    indicator,
    territory,
    submesh=MeshLevel.Region,
    include_place_names=None,
    flows=False,
):
    mapped_mesh = "DEPCOM" if territory.mesh == "com" else territory.mesh.upper()
    mapped_submesh = "DEPCOM" if submesh == "com" else submesh.upper()

    geo_id_values = "', '".join([id.strip() for id in territory.id.split(",")])

    table_prefix = (
        indicator.flows_db_table_prefix if flows else indicator.db_table_prefix
    )

    arbo_sub_query = f"""
    SELECT DISTINCT("{mapped_submesh}")
    FROM arborescence_geo arbo
    WHERE arbo."{mapped_mesh}" in('{geo_id_values}')
    """

    where = (
        f"""
        WHERE (indic.code_{submesh.lower()}_1 in ({arbo_sub_query})
        OR indic.code_{submesh.lower()}_2 in ({arbo_sub_query}))
        """
        if flows
        else f"""
        WHERE indic.code_{submesh.lower()} in ({arbo_sub_query})
        """
    )

    return f"""
    "{table_prefix}_{submesh.lower()}" indic
    {get_place_names_join(include_place_names, mapped_submesh, submesh, flows)}
    {where}
    """


def calculate_aggregate_values(indicator, with_alternative=True):
    # TODO coverage tester avec un indicateur de cette sorte
    # Bastien doit exporter des nouvelles données pour faire le test
    if not indicator.is_composite:
        return "SUM(valeur) as valeur"

    # TODO coverage tester avec un indicateur de cette sorte
    # Bastien doit exporter des nouvelles données pour faire le test
    if indicator.aggregation_function == AggregationFunctions.DISCRETE_COMPONENT_2:
        sql = f"SUM(composante_1) / COALESCE(NULLIF(SUM(composante_2), 0), 1) * {indicator.aggregation_constant} as valeur"
        if with_alternative:
            sql += ", SUM(composante_1) as valeur_alternative"
        return sql
    breakdown_dimension = get_breakdown_dimension(indicator)
    breakdown_count = (
        f" * COUNT(DISTINCT({breakdown_dimension.db_name})) "
        if breakdown_dimension
        else ""
    )
    sql = f"SUM(composante_1) / COALESCE(NULLIF(SUM(composante_2), 0), 1) {breakdown_count} * {indicator.aggregation_constant} as valeur"
    if with_alternative:
        sql += ", SUM(composante_1) as valeur_alternative"
    return sql


def get_territories_ids(main_territory_codes, territory_mesh, submesh):
    mapped_territory_mesh = (
        "DEPCOM" if territory_mesh == "com" else territory_mesh.upper()
    )
    mapped_submesh = "DEPCOM" if submesh == "com" else submesh.upper()

    query = f"""
        SELECT DISTINCT "{mapped_submesh}" as code
        FROM arborescence_geo
        WHERE "{mapped_territory_mesh}" IN ('{"', '".join(main_territory_codes)}')
        """

    territories_ids = [r["code"] for r in run_custom_query(query)]
    return territories_ids


def get_sub_territories(territory, submesh):
    territory_codes = territory.id.split(",")
    mapped_territory_mesh = (
        "DEPCOM" if territory.mesh == "com" else territory.mesh.upper()
    )
    mapped_submesh = "DEPCOM" if submesh == "com" else submesh.upper()

    query = f"""
        SELECT DISTINCT "{mapped_submesh}" as code, "NOM_{mapped_submesh}" as name
        FROM arborescence_geo
        WHERE "{mapped_territory_mesh}" IN ('{"', '".join(territory_codes)}')
        """

    return run_custom_query(query)


def get_where_territory(territory):
    territory_id = (
        FRANCE_DB_VALUES[territory.id]
        if territory.mesh == MeshLevel.National
        else territory.id
    )
    return f""" "code_{territory.mesh}" = '{territory_id}' """


def get_values_for_territory(indicator, territory, filters=None):
    value = calculate_aggregate_values(indicator)
    where_territory = get_where_territory(territory)
    query = f"""
        SELECT {value}, annee
        FROM "{indicator.db_table_prefix}_{territory.mesh}" as indic
        WHERE {where_territory}
        {add_optional_filters(indicator, filters)}
        GROUP BY annee
        ORDER BY annee DESC
    """
    return query


def get_territory_name(territory):
    territory_mesh = "DEPCOM" if territory.mesh == "com" else territory.mesh.upper()
    query = f"""SELECT "NOM_{territory_mesh}" as nom FROM arborescence_geo WHERE "{territory_mesh}" = '{territory.id}' LIMIT 1;"""
    results = run_custom_query(query)
    return results[0]["nom"] if results else ""



import base64
import os
import subprocess
from tempfile import NamedTemporaryFile

from django.conf import settings
from markdown import markdown


def _html_to_pdf(html_content, output_pdf_path):
    try:
        process = subprocess.Popen(
            [
                "wkhtmltopdf",
                "-",
                output_pdf_path,
            ],  # '-' tells wkhtmltopdf to read from stdin
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        stdout, stderr = process.communicate(input=html_content.encode("utf-8"))
        if process.returncode != 0:
            print("Error:", stderr.decode("utf-8"))
        else:
            print(f"PDF successfully created at: {output_pdf_path}")
    except FileNotFoundError:
        print("Error: wkhtmltopdf not found. Ensure it is installed and in your PATH.")
    except Exception as e:
        print(f"An error occurred: {e}")


def _generate_pdf_from_methodo(indicator, output_path):
    html = markdown(indicator.methodo)
    relative_logo_path = "whitenoise_root/ministere_logo.png"
    logo_path = os.path.join(settings.BASE_DIR, relative_logo_path)
    with open(logo_path, "rb") as fd:
        encoded_logo = base64.b64encode(fd.read()).decode("utf-8")
    html = (
        """
    <!DOCTYPE html>
    <html lang="fr">
    <head>
        <meta charset="UTF-8">
        <title>Accents Test</title>
        <style>
            body {
                font-family: Tahoma, Arial, sans-serif;
            }
        </style>
    </head>
    <body>
    <header>
        <table>
            <tr>
                <td>
                """
        + f'<img width="150px" src="data:image/png;base64,{encoded_logo}"/>'
        + """
                </td>
                <td style="padding-left: 64px;">
                    <div style="font-size: 30px; font-weight: 600; margin-bottom: 8px;">Tableau de bord des mobilités durables</div>
                    <div>Fiche méthodologique</div>
                </td>
            </tr>
        </table>
    </header>
    <main>
    """
        + f"<h1>{indicator.title}</h1>"
        + f"<p>Thématique : {indicator.sub_theme.theme.title} / {indicator.sub_theme.title}</p>"
        + html
        + """
    </main>
    </body>
    </html>
    """
    )
    _html_to_pdf(html, output_path)


def reset_methodo_file(indicator):
    # Create a temporary file for the PDF
    with NamedTemporaryFile(delete=False, suffix=".pdf") as temp_file:
        pdf_path = temp_file.name

    # Generate the PDF
    _generate_pdf_from_methodo(indicator, pdf_path)

    # Read the binary content of the generated PDF file
    with open(pdf_path, "rb") as pdf_file:
        pdf_content = pdf_file.read()

    # Save the binary content to the BinaryField
    indicator.methodo_file = pdf_content
    indicator.save()

    # Clean up temporary file
    os.remove(pdf_path)



from django.urls import path

from .views import (
    comparison_histogram_export_view,
    comparison_histogram_view,
    download_indicator_methodo_view,
    flows_view,
    indicator_details_table_export_view,
    indicator_details_table_view,
    indicator_histogram_export_view,
    indicator_histogram_view,
    indicator_proportions_export_view,
    indicator_statistics_view,
    indicator_submesh_territories_view,
    indicator_top_10_export_view,
    indicator_top_10_view,
    indicator_values_export_view,
    indicator_values_view,
    proportions_chart_view,
)

app_name = "indicators-api"

urlpatterns = [
    path("flows/", flows_view, name="flows"),
    path(
        "<str:name>/methodo/",
        download_indicator_methodo_view,
        name="download-indicator-methodo",
    ),
    path(
        "<str:name>/statistics/",
        indicator_statistics_view,
        name="statistics",
    ),
    path(
        "<str:name>/values/export/",
        indicator_values_export_view,
        name="values-export",
    ),
    path(
        "<str:name>/values/",
        indicator_values_view,
        name="values",
    ),
    path(
        "<str:name>/histogram/export/",
        indicator_histogram_export_view,
        name="histogram-export",
    ),
    path(
        "<str:name>/histogram/",
        indicator_histogram_view,
        name="histogram",
    ),
    path(
        "<str:name>/top-10/export/",
        indicator_top_10_export_view,
        name="top-10-export",
    ),
    path(
        "<str:name>/top-10/",
        indicator_top_10_view,
        name="top-10",
    ),
    path(
        "<str:name>/details/table/",
        indicator_details_table_view,
        name="details-table",
    ),
    path(
        "<str:name>/details/table/export/",
        indicator_details_table_export_view,
        name="details-table-export",
    ),
    path(
        "<str:name>/comparison-histogram/export/",
        comparison_histogram_export_view,
        name="comparison-histogram-export",
    ),
    path(
        "<str:name>/comparison-histogram/",
        comparison_histogram_view,
        name="comparison-histogram",
    ),
    path(
        "<str:name>/proportions/export/",
        indicator_proportions_export_view,
        name="proportions-export",
    ),
    path(
        "<str:name>/proportions/",
        proportions_chart_view,
        name="proportions",
    ),
    path("<str:name>/submesh/", indicator_submesh_territories_view, name="submesh"),
]



from collections import defaultdict

from .commons import (
    add_optional_filters,
    calculate_aggregate_values,
    get_last_year,
    get_table_data_for_geography,
)
from .utils import get_breakdown_dimension, run_custom_query


def breakdown_territories(indicator, submesh, year, filters, top_10_territories):
    territories_code = ", ".join(
        [f"'{territory['code_geo']}'" for territory in top_10_territories]
    )
    breakdown_dimension = get_breakdown_dimension(indicator).db_name

    query = f"""
        SELECT {calculate_aggregate_values(indicator, with_alternative=False)},
            "{breakdown_dimension}" as dimension,
            "code_{submesh}" as code_geo
        FROM "{indicator.db_table_prefix}_{submesh}" as indic
        WHERE code_{submesh} in ({territories_code}) AND annee = {year}
        {add_optional_filters(indicator, filters)}
        GROUP BY "code_{submesh}", "{breakdown_dimension}"
    """
    results = run_custom_query(query)
    return results


def get_top_10_territories(indicator, territory, submesh, year, filters):
    query = f"""
    SELECT {calculate_aggregate_values(indicator, with_alternative=False)},
    indic.code_{submesh} as code_geo, arbo.lieu as lieu
    FROM {get_table_data_for_geography(indicator, territory, submesh, include_place_names=True)}
    {add_optional_filters(indicator, filters)}
    AND annee = {year}
    GROUP BY code_{submesh}, arbo.lieu
    ORDER BY valeur DESC
    LIMIT 10
    """
    results = run_custom_query(query)
    return results


def get_indicator_top_10_data(indicator, territory, submesh, filters):
    indicator_details = {}

    breakdown_dimension = get_breakdown_dimension(indicator)
    breakdown_dimension_name = (
        breakdown_dimension.db_name if breakdown_dimension else None
    )
    breakdown_filters = filters.get(breakdown_dimension_name, [])

    last_year = get_last_year(indicator, submesh)

    territories = get_top_10_territories(
        indicator, territory, submesh, last_year, filters
    )

    if breakdown_dimension:
        breakdown = breakdown_territories(
            indicator, submesh, last_year, filters, territories
        )
        breakdown_by_geocode = defaultdict(dict)
        for row in breakdown:
            breakdown_by_geocode[row["code_geo"]][row["dimension"]] = row["valeur"]
        filters_color = {f.db_name: f.color for f in breakdown_dimension.filters.all()}
        datasets_top_bar_chart = [
            {
                "label": f,
                "data": [
                    breakdown_by_geocode[territory["code_geo"]][f]
                    for territory in territories
                ],
                "color": filters_color.get(f),
            }
            for f in breakdown_filters
        ]
    # TODO coverage faire un test avec un indicateur qui n'a pas de dimensions
    # necessite un export de données de Bastien
    else:
        datasets_top_bar_chart = [
            {
                "label": indicator.unite,
                "data": [territory["valeur"] for territory in territories],
            }
        ]

    labels_top_bar_chart = [territory["lieu"] for territory in territories]

    csv_data = []
    for territory in territories:
        csv_row = {}
        csv_row["Territoire"] = territory["lieu"]
        csv_row["Code Géographique"] = territory["code_geo"]
        csv_row[f"Valeur {indicator.unite}"] = territory["valeur"]
        if breakdown_dimension:
            for filter in breakdown_filters:
                csv_row[filter] = breakdown_by_geocode[territory["code_geo"]][filter]
        csv_data.append(csv_row)

    indicator_details["labelsTopBarChart"] = labels_top_bar_chart
    indicator_details["datasetsTopBarChart"] = datasets_top_bar_chart

    return indicator_details, csv_data



from django.db import models

from territories_dashboard_lib.commons.models import CommonModel
from territories_dashboard_lib.indicators_lib.models import Indicator

from .enums import GeoFeatureType


class GeoFeature(CommonModel):
    name = models.TextField(
        verbose_name="Nom de la table en base",
        help_text="attention le nom doit être exactement le même que celui en base.",
    )
    title = models.TextField(verbose_name="Titre à afficher", default="à compléter")
    unite = models.TextField(verbose_name="Unité au pluriel")
    indicator = models.ForeignKey(
        Indicator,
        related_name="geo_features",
        on_delete=models.CASCADE,
        help_text="Cette donnée géographique sera ajoutée à la carte de l'indicateur.",
    )
    geo_type = models.TextField(
        choices=GeoFeatureType.choices,
        default=GeoFeatureType.point,
        verbose_name="Type de géométrie",
    )
    point_icon_svg = models.TextField(
        blank=True,
        null=True,
        verbose_name="Icone des points",
        help_text="Icone SVG a utilisé pour les points.<br/>Laisser vide pour les autres types de géométrie.<br/>Veuillez charger un fichier provenant des <a href='/'>Material Icons</a> de Google.",
    )
    color = models.TextField(
        default="#000000",
        verbose_name="Couleur",
        help_text="Couleur des données sur la date, à renseigner au format hexadécimal, par exemple: '#05B2F9'",
    )
    show_on_fr_level = models.BooleanField(
        default=True,
        verbose_name="Afficher au niveau France entière",
        help_text="Décochez si la quantité de données est trop importante pour le niveau France entière.",
    )

    def __str__(self):
        return self.name

    class Meta:
        ordering = ("indicator", "name")
        verbose_name = "Données de la carte"


class GeoElement(CommonModel):
    geo_feature = models.ForeignKey(
        GeoFeature, related_name="items", on_delete=models.CASCADE
    )
    name = models.TextField(verbose_name="Nom de la colonne en DB")
    label = models.TextField()
    filterable = models.BooleanField(default=True, verbose_name="Filtrable")



from typing import Annotated

from pydantic import StringConstraints

TerritoryCode = Annotated[
    str, StringConstraints(strip_whitespace=True, pattern=r"^[A-Za-z0-9]*$")
]

# comme un TerritoryCode mais prend en compte l'exception pour la France entière : territory=FR0,FR1,FR2-fr
MutualisedTerritoryCode = Annotated[
    str, StringConstraints(strip_whitespace=True, pattern=r"^[A-Za-z0-9,]*$")
]

SQlName = Annotated[
    str, StringConstraints(strip_whitespace=True, pattern=r"^[A-Za-z0-9_]*$")
]

SimpleIDType = Annotated[
    str, StringConstraints(strip_whitespace=True, pattern=r"^[A-Za-z0-9_\-]*$")
]



from functools import wraps

from territories_dashboard_lib.indicators_lib.enums import (
    FRANCE_GEOLEVEL_TITLES,
    MESH_TITLES,
    FranceGeoLevel,
    MeshLevel,
    get_all_meshes,
    get_allow_same_mesh,
)
from territories_dashboard_lib.indicators_lib.query.utils import run_custom_query

TERRITORY_DEFAULT = {
    "id": FranceGeoLevel.METRO,
    "mesh": MeshLevel.National,
    "name": FRANCE_GEOLEVEL_TITLES[FranceGeoLevel.METRO],
}

CMP_TERRITORY_DEFAULT = {
    "id": FranceGeoLevel.All,
    "mesh": MeshLevel.National,
    "name": FRANCE_GEOLEVEL_TITLES[FranceGeoLevel.All],
}

TERRITORY_PARAM_NAME = "territory"
CMP_TERRITORY_PARAM_NAME = "cmp-territory"
MESH_PARAM_NAME = "mesh"
ONE_WEEK_IN_SECONDS = 604800


class BadParam(Exception):
    pass


# TODO coverage
# appeler la page détail d'un indicateur (page html, pas api) avec plus de combinaisons de paramètres GET
# tester les valeurs du context du template
class ParamsHandler:
    def __init__(self, request):
        self.request = request
        self.territory_id = None
        self.territory_mesh = None
        self.territory_name = None
        self.cmp_territory_id = None
        self.cmp_territory_mesh = None
        self.cmp_territory_name = None
        self.mesh = None
        self.meshes = []
        self.comparison = request.resolver_match.view_name == "website:comparison"

    ######################## Territory

    def parse_territory(self, territory):
        if not territory:
            raise BadParam
        parts = territory.split("-")
        allowed_chars = set(
            "0123456789AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz, "
        )
        if (
            len(parts) != 2
            or not all(char in allowed_chars for char in parts[0])
            or parts[1] not in MeshLevel
        ):
            raise BadParam
        return parts[0], parts[1]

    def get_territory_name(self, territory_id, territory_mesh):
        if territory_mesh == MeshLevel.National:
            return FRANCE_GEOLEVEL_TITLES[territory_id]
        mesh = "DEPCOM" if territory_mesh == "com" else territory_mesh.upper()
        query = f"""
        SELECT DISTINCT "NOM_{mesh}" AS name FROM arborescence_geo
        WHERE "{mesh}" = '{territory_id}'
        """
        try:
            results = run_custom_query(query)
        except Exception:
            raise BadParam
        if not results:
            raise BadParam
        return results[0]["name"]

    def get_territory_default(self, cookie_territory):
        try:
            territory_id, territory_mesh = self.parse_territory(cookie_territory)
            territory_name = self.get_territory_name(territory_id, territory_mesh)
        except BadParam:
            return (
                TERRITORY_DEFAULT["id"],
                TERRITORY_DEFAULT["mesh"],
                TERRITORY_DEFAULT["name"],
            )
        return territory_id, territory_mesh, territory_name

    def choose_territory(self):
        url_territory = self.request.GET.get(TERRITORY_PARAM_NAME)
        try:
            territory_id, territory_mesh = self.parse_territory(url_territory)
            self.territory_name = self.get_territory_name(territory_id, territory_mesh)
            self.territory_id = territory_id
            self.territory_mesh = territory_mesh
        except BadParam:
            cookie_territory = self.request.COOKIES.get(TERRITORY_PARAM_NAME)
            self.territory_id, self.territory_mesh, self.territory_name = (
                self.get_territory_default(cookie_territory)
            )

    def construct_territory_value(self):
        return f"{self.territory_id}-{self.territory_mesh}"

    def territory_is_not_default(self):
        return (
            self.territory_id != TERRITORY_DEFAULT["id"]
            or self.territory_mesh != TERRITORY_DEFAULT["mesh"]
        )

    ######################## Compared Territory

    def choose_cmp_territory(self):
        url_territory = self.request.GET.get(CMP_TERRITORY_PARAM_NAME)
        try:
            territory_id, territory_mesh = self.parse_territory(url_territory)
            self.cmp_territory_name = self.get_territory_name(
                territory_id, territory_mesh
            )
            self.cmp_territory_id = territory_id
            self.cmp_territory_mesh = territory_mesh
        except BadParam:
            cookie_territory = self.request.COOKIES.get(CMP_TERRITORY_PARAM_NAME)
            self.cmp_territory_id, self.cmp_territory_mesh, self.cmp_territory_name = (
                self.get_territory_default(cookie_territory)
            )

    def construct_cmp_territory_value(self):
        return f"{self.cmp_territory_id}-{self.cmp_territory_mesh}"

    def cmp_territory_is_not_default(self):
        return (
            self.cmp_territory_id != CMP_TERRITORY_DEFAULT["id"]
            or self.cmp_territory_mesh != CMP_TERRITORY_DEFAULT["mesh"]
        )

    ######################## Mesh

    def get_max_territory_mesh(self):
        meshes = get_all_meshes()
        if self.comparison is False:
            return self.territory_mesh
        if meshes.index(self.cmp_territory_mesh) > meshes.index(self.territory_mesh):
            return self.cmp_territory_mesh
        return self.territory_mesh

    def is_not_valid_mesh(self, mesh):
        if mesh is None:
            return True
        meshes = get_all_meshes()
        max_territory_mesh = self.get_max_territory_mesh()
        allow_same_mesh = get_allow_same_mesh()
        is_not_valid = (
            meshes.index(max_territory_mesh) > meshes.index(mesh)
            if allow_same_mesh
            else meshes.index(max_territory_mesh) >= meshes.index(mesh)
        )
        return is_not_valid

    def get_default_mesh(self):
        max_territory_mesh = self.get_max_territory_mesh()
        meshes = get_all_meshes()
        if meshes.index(max_territory_mesh) == len(meshes) - 1:
            default_mesh = meshes[-1]
        else:
            mesh_index = meshes.index(max_territory_mesh)
            if get_allow_same_mesh() is False:
                mesh_index += 1
            default_mesh = meshes[mesh_index]
        return default_mesh

    def get_cookie_mesh(self):
        value = self.request.COOKIES.get(MESH_PARAM_NAME)
        if not value:
            return None
        parts = value.split("-")
        if len(parts) != 2 or parts[0] not in MeshLevel or parts[1] not in MeshLevel:
            return None
        if self.territory_mesh != parts[0]:
            return None
        return parts[1]

    def choose_mesh(self):
        url_mesh = self.request.GET.get(MESH_PARAM_NAME)
        url_mesh = url_mesh if url_mesh in MeshLevel else None
        if self.is_not_valid_mesh(url_mesh):
            cookie_mesh = self.get_cookie_mesh()
            if self.is_not_valid_mesh(cookie_mesh):
                self.mesh = self.get_default_mesh()
            else:
                self.mesh = cookie_mesh
        else:
            self.mesh = url_mesh
        max_territory_mesh = self.get_max_territory_mesh()
        meshes = [
            m
            for m in get_all_meshes()
            if not (m == MeshLevel.Town and max_territory_mesh == MeshLevel.National)
        ]
        min_mesh_index = meshes.index(max_territory_mesh)
        if get_allow_same_mesh() is False:
            min_mesh_index += 1
        self.meshes = meshes[min(min_mesh_index, len(meshes) - 1) :]

    ######################## Commons

    def set_cookie(self, response):
        if self.territory_is_not_default():
            response.set_cookie(
                key=TERRITORY_PARAM_NAME,
                value=self.construct_territory_value(),
                max_age=ONE_WEEK_IN_SECONDS,
            )
        else:
            response.delete_cookie(TERRITORY_PARAM_NAME)
        if self.cmp_territory_is_not_default():
            response.set_cookie(
                key=CMP_TERRITORY_PARAM_NAME,
                value=self.construct_cmp_territory_value(),
                max_age=ONE_WEEK_IN_SECONDS,
            )
        else:
            response.delete_cookie(CMP_TERRITORY_PARAM_NAME)
        if self.mesh != self.get_default_mesh():
            response.set_cookie(
                key=MESH_PARAM_NAME,
                value=f"{self.territory_mesh}-{self.mesh}",
                max_age=ONE_WEEK_IN_SECONDS,
            )
        else:
            response.delete_cookie(MESH_PARAM_NAME)

    def add_to_context(self, context):
        url_params = []
        if self.territory_is_not_default():
            url_params.append(
                f"{TERRITORY_PARAM_NAME}={self.construct_territory_value()}"
            )
        if self.comparison and self.cmp_territory_is_not_default():
            url_params.append(
                f"{CMP_TERRITORY_PARAM_NAME}={self.construct_cmp_territory_value()}"
            )
        if self.mesh != self.get_default_mesh():
            url_params.append(f"{MESH_PARAM_NAME}={self.mesh}")
        context["params"] = {
            "territory_id": self.territory_id,
            "territory_mesh": self.territory_mesh,
            "territory_name": self.territory_name,
            "cmp_territory_id": self.cmp_territory_id,
            "cmp_territory_mesh": self.cmp_territory_mesh,
            "cmp_territory_name": self.cmp_territory_name,
            "mesh": self.mesh,
            "meshes": self.meshes,
            "meshes_titles": MESH_TITLES,
            "all_meshes": get_all_meshes(),
            "url_params": "&".join(url_params),
        }


def with_params(view_func):
    @wraps(view_func)
    def wrapper(request, *args, **kwargs):
        handler = ParamsHandler(request)
        handler.choose_territory()
        handler.choose_cmp_territory()
        handler.choose_mesh()

        context = kwargs.get("context", {})
        handler.add_to_context(context)

        response = view_func(request, *args, context=context, **kwargs)
        handler.set_cookie(response)
        return response

    return wrapper



from django import forms
from django.forms import Textarea

from territories_dashboard_lib.website_lib.models import (
    GlossaryItem,
    LandingPage,
    MainConf,
    StaticPage,
)


class MainConfAdminForm(forms.ModelForm):
    class Meta:
        model = MainConf
        fields = "__all__"
        widgets = {
            "title": Textarea(attrs={"rows": 1, "cols": 30}),
        }


class GlossaryItemAdminForm(forms.ModelForm):
    class Meta:
        model = GlossaryItem
        fields = "__all__"
        widgets = {
            "word": Textarea(attrs={"rows": 1, "cols": 30}),
        }


class LandingPageAdminForm(forms.ModelForm):
    class Meta:
        model = LandingPage
        fields = "__all__"
        widgets = {
            "title": Textarea(attrs={"rows": 1, "cols": 30}),
            "button_link": Textarea(attrs={"rows": 1, "cols": 30}),
        }


class StaticPageAdminForm(forms.ModelForm):
    class Meta:
        model = StaticPage
        fields = "__all__"
        widgets = {
            "name": Textarea(attrs={"rows": 1, "cols": 30}),
            "url": Textarea(attrs={"rows": 1, "cols": 30}),
        }



from pydantic import BaseModel

from territories_dashboard_lib.tracking_lib.enums import EventType


class EventPayload(BaseModel):
    indicator: str
    event: EventType
    objet: str
    type: str



from django.conf import settings
from django.db import models


class AggregationFunctions(models.TextChoices):
    DISCRETE_COMPONENT_2 = "discrete"
    REPEATED_COMPONENT_2 = "repeated"


class MeshLevel(models.TextChoices):
    National = "fr"
    Region = "reg"
    Department = "dep"
    Epci = "epci"
    Town = "com"


class GeoLevel(models.TextChoices):
    France = "fr"
    Region = "reg"
    Department = "dep"
    Epci = "epci"
    Town = "com"


class FranceGeoLevel(models.TextChoices):
    All = "FR0,FR1,FR2"
    METRO = "FR0,FR1"
    METRO_HORS_IDF = "FR0"


FRANCE_GEOLEVEL_TITLES = {
    FranceGeoLevel.All: "France entière",
    FranceGeoLevel.METRO: "France métropolitaine",
    FranceGeoLevel.METRO_HORS_IDF: "France métropolitaine hors IDF",
}

FRANCE_DB_VALUES = {
    FranceGeoLevel.All: "FR_TOT",
    FranceGeoLevel.METRO: "FR_METRO",
    FranceGeoLevel.METRO_HORS_IDF: "FR_METRO_HORS_IDF",
}


DEFAULT_MESH = MeshLevel.Region

MESH_TITLES = {
    MeshLevel.National: "France entière",
    MeshLevel.Region: "Région",
    MeshLevel.Department: "Département",
    MeshLevel.Epci: "Intercommunalité",
    MeshLevel.Town: "Commune",
}

MESH_DB = {
    MeshLevel.Region: "REG",
    MeshLevel.Department: "DEP",
    MeshLevel.Epci: "EPCI",
    MeshLevel.Town: "DEPCOM",
}


def get_miminum_mesh():
    try:
        town_mesh_is_disabled = settings.DISABLE_TOWN_MESH
        return MeshLevel.Epci if town_mesh_is_disabled else MeshLevel.Town
    except AttributeError:
        return MeshLevel.Town


def get_allow_same_mesh():
    try:
        return bool(settings.ALLOW_SAME_MESH)
    except AttributeError:
        return False


def get_all_meshes():
    min_mesh = get_miminum_mesh()
    meshes = []
    for m in MeshLevel:
        meshes.append(m)
        if m == min_mesh:
            break
    return meshes



# Generated by Django 5.2.3 on 2025-06-19 07:13

import django.db.models.deletion
import martor.models
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SubTheme',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=64, unique=True)),
                ('index_in_theme', models.IntegerField(default=0)),
                ('title', models.CharField(max_length=128)),
                ('description', models.TextField(blank=True, default='')),
            ],
            options={
                'verbose_name': '2 - Sous-thème',
                'ordering': ('theme', 'index_in_theme'),
            },
        ),
        migrations.CreateModel(
            name='Indicator',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('is_active', models.BooleanField(default=True)),
                ('index_in_theme', models.IntegerField(default=0)),
                ('name', models.CharField(max_length=32, unique=True)),
                ('title', models.CharField(max_length=128)),
                ('db_table_prefix', models.CharField(max_length=128)),
                ('is_composite', models.BooleanField(default=False)),
                ('show_alternative', models.BooleanField(default=True, help_text='Pour certains indicateurs composites (par exemple les moyennes) on ne veut pas afficher la valeur alternative.', verbose_name='Afficher la valeur alternative')),
                ('aggregation_constant', models.DecimalField(decimal_places=5, default=1, max_digits=10)),
                ('aggregation_function', models.CharField(choices=[('discrete', 'Discrete Component 2'), ('repeated', 'Repeated Component 2')], default='repeated', max_length=8)),
                ('unite', models.CharField(max_length=32)),
                ('unite_nom_accessible', models.CharField(blank=True, default='', help_text="Nom accessible de l'unité qui sera lu par le lecteur d'écran.", max_length=64)),
                ('unite_alternative', models.CharField(blank=True, default=None, max_length=32, null=True)),
                ('unite_alternative_nom_accessible', models.CharField(blank=True, default='', help_text="Nom accessible de l'unité alternative qui sera lu par le lecteur d'écran.", max_length=64)),
                ('flows_db_table_prefix', models.CharField(blank=True, help_text='Les données des flux seront affichés sur la carte et le graphique de Sankey.', max_length=128, null=True, verbose_name='Table des flux (prefix)')),
                ('flows_dimension', models.CharField(blank=True, help_text='Nom de la colonne des dimensions de la table des flux', max_length=32, null=True, verbose_name='Dimension des flux')),
                ('show_evolution', models.BooleanField(default=False)),
                ('source', models.TextField(blank=True, default='')),
                ('description', models.TextField(blank=True, default='')),
                ('methodo', martor.models.MartorField(blank=True, default='')),
                ('methodo_file', models.BinaryField(blank=True, null=True)),
                ('secondary_indicator', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='indicators_lib.indicator', verbose_name='Second indicateur à afficher sur la carte')),
                ('sub_theme', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='indicators', to='indicators_lib.subtheme')),
            ],
            options={
                'verbose_name': '3 - Indicateur',
                'ordering': ('sub_theme', 'index_in_theme'),
            },
        ),
        migrations.CreateModel(
            name='Dimension',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('db_name', models.TextField()),
                ('title', models.TextField()),
                ('is_breakdown', models.BooleanField(default=False, help_text="Dans le cas de plusieurs dimensions pour un indicateur, l'une d'entre elles doit être la dimension de réparition pour les graphiques.", verbose_name='Répartir selon cette dimension')),
                ('indicator', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='dimensions', to='indicators_lib.indicator')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Theme',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('ordering', models.IntegerField(default=0)),
                ('name', models.CharField(max_length=64, unique=True)),
                ('title', models.CharField(max_length=128)),
                ('objectif_theme', models.TextField(blank=True)),
                ('action_theme', models.TextField(blank=True)),
            ],
            options={
                'verbose_name': '1 - Thème',
                'ordering': ('ordering',),
                'indexes': [models.Index(fields=['name'], name='indicators__name_fea408_idx')],
            },
        ),
        migrations.AddField(
            model_name='subtheme',
            name='theme',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='sub_themes', to='indicators_lib.theme'),
        ),
        migrations.CreateModel(
            name='Filter',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('db_name', models.CharField(max_length=128)),
                ('order', models.IntegerField(default=0)),
                ('default', models.BooleanField(default=True)),
                ('dimension', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='filters', to='indicators_lib.dimension')),
            ],
            options={
                'verbose_name': 'Filtre',
                'ordering': ('dimension', 'order', 'db_name'),
                'indexes': [models.Index(fields=['db_name'], name='indicators__db_name_d32459_idx'), models.Index(fields=['dimension', 'order'], name='indicators__dimensi_aa3fdf_idx')],
                'unique_together': {('dimension', 'db_name')},
            },
        ),
        migrations.AddIndex(
            model_name='indicator',
            index=models.Index(fields=['sub_theme'], name='indicators__sub_the_b49caf_idx'),
        ),
        migrations.AddIndex(
            model_name='indicator',
            index=models.Index(fields=['name'], name='indicators__name_c5606a_idx'),
        ),
        migrations.AddIndex(
            model_name='subtheme',
            index=models.Index(fields=['theme', 'index_in_theme'], name='indicators__theme_i_0cbbc1_idx'),
        ),
        migrations.AddIndex(
            model_name='subtheme',
            index=models.Index(fields=['name'], name='indicators__name_79c542_idx'),
        ),
    ]



# Generated by Django 5.2.6 on 2025-09-23 12:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('website_lib', '0003_alter_mainconf_footer_navigation_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='mainconf',
            name='description',
            field=models.TextField(blank=True, help_text='Utilisé pour la balise meta description du site et pour le relais sur les réseaux sociaux.', null=True, verbose_name='Description du site'),
        ),
        migrations.AddField(
            model_name='mainconf',
            name='social_image_url',
            field=models.TextField(blank=True, help_text="Lien d'une image (idéalement 1200x630px). Elle sera utilisée lors du partage sur les réseaux sociaux.", null=True, verbose_name='Image de description'),
        ),
    ]



from .commons import generate_aggregate_query_for_location
from .utils import run_custom_query


def generate_min_max_queries(indicator):
    result = """
    , min_value AS (SELECT MIN(valeur) AS min FROM aggregat),
    max_value AS (SELECT MAX(valeur) AS max FROM aggregat)
    """
    if indicator.is_composite:
        result += """
        , min_alternative_value AS (SELECT MIN(valeur_alternative) AS min_alternative FROM aggregat),
        max_alternative_value AS (SELECT MAX(valeur_alternative) AS max_alternative FROM aggregat)
        """
    return result


def get_min_max_statistics(indicator, submesh):
    result = f"""
    {get_values_for("min", submesh)}
    {get_values_for("max", submesh)}
    """
    if indicator.is_composite:
        result += f"""
        {get_values_for("min_alternative", submesh, True)}
        {get_values_for("max_alternative", submesh, True)}
        """
    return result


def get_values_for(extremum, submesh, is_alternative=False):
    return f"""
    {extremum}_value,
    (
        SELECT code_{submesh} AS code_{extremum}
        FROM aggregat
        WHERE valeur{"_alternative" if is_alternative else ""} = (SELECT * FROM {extremum}_value) LIMIT 1
    ) AS code_{extremum},
    (
        SELECT count(code_{submesh}) AS count_{extremum}
        FROM aggregat
        WHERE valeur{"_alternative" if is_alternative else ""} = (SELECT * FROM {extremum}_value)
    ) AS count_{extremum},
    """


def get_med_values(indicator):
    return f"""
    (SELECT
    PERCENTILE_DISC(0.5) WITHIN GROUP (order by valeur) AS med
    {", PERCENTILE_DISC(0.5) WITHIN GROUP (order by valeur_alternative) AS med_alternative" if indicator.is_composite else ""}
    FROM aggregat) AS meds
    """


def get_geography_statistics_values_for_indicator(
    indicator, territory, submesh, filters
):
    query = f"""
    {generate_aggregate_query_for_location(indicator, territory, submesh, filters)}
    {generate_min_max_queries(indicator)}

    SELECT * FROM
    {get_min_max_statistics(indicator, submesh)}
    {get_med_values(indicator)}
    """

    return query


def get_names_from_codes(dict_result, submesh):
    codes_to_fetch = []
    for key, value in dict_result.items():
        if key.startswith("code_"):
            codes_to_fetch.append(value)
    mapped_submesh = "DEPCOM" if submesh == "com" else submesh.upper()
    query = f"""
    SELECT DISTINCT "{mapped_submesh}" as code, "NOM_{mapped_submesh}" as name
    FROM arborescence_geo
    WHERE "{mapped_submesh}" IN ({", ".join(["'" + c + "'" for c in codes_to_fetch])});
    """
    rows = run_custom_query(query)
    names_dict = {}
    for row in rows:
        names_dict[row["code"]] = row["name"]
    updated_dict = {}
    for key, value in dict_result.items():
        updated_dict[key] = value
        if key.startswith("code_"):
            updated_dict[key + "_name"] = names_dict[value]
    return updated_dict
